#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include "syserr.h"
#include "daemonize.h"

static int quit;

static void sigint(int signo)
{
  quit = signo;
}

int main(int argc, char *argv[])
{
  unsigned long long size, i;
  long page_size, phys_pages, percent = 90;
  char *base;
  int debug = 0, c;
  extern int optind;

  while ((c = getopt(argc, argv, "d")) != -1) {
    switch (c) {
    case 'd':
      debug = 1;
      break;
    }
  }

  if (!debug)
    daemonize(argv[0]);

  SYSERR2(signal(SIGINT, sigint), SIG_ERR);

  SYSERR(page_size = sysconf(_SC_PAGESIZE));
  SYSERR(phys_pages = sysconf(_SC_PHYS_PAGES));

  if (argc > 1)
    percent = strtoul(argv[optind], NULL, 10);

  phys_pages *= percent;
  phys_pages /= 100;

  size = page_size;
  size *= phys_pages; /* avoid overflow */

  SYSERR2(base = mmap(NULL, size, PROT_READ | PROT_WRITE, 
		      MAP_PRIVATE | MAP_ANONYMOUS | MAP_LOCKED, -1, 0),
	  MAP_FAILED);

  for (i = 0; i < size && !quit; i += page_size)
    base[i] = 0;  /* touch every page to fault it in */

  while (!quit)
    sleep(100);

  warning("Caught signal %d, exiting", quit);

  return 0;
}
